#include <ctype.h>

#include <gtk/gtk.h>
#include <libnotify/notify.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/wait.h>

#include <glib.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib-lowlevel.h>

struct wimax_conf{
	int nspid;
	int use_pkm;
	int eap_type;
	int cert_null;
	int cert_nv;
	
	gchar *anonymous_identity;
	gchar *identity;
	gchar *password;
	gchar *ca_cert;
	gchar *client_cert;
	gchar *private_key;
	gchar *private_key_passwd;
	
	int wimax_verbose_level;
	int wpa_debug_level;
	gchar *log_file;
	gchar *event_script;
};


struct wimax_cm {
	int child_pid;
	struct wimax_conf *conf;
	GKeyFile *keyfile;
	gchar *cfgfile;
	
    GtkStatusIcon *tray_icon;
	NotifyNotification *tray_notify;
	GtkWidget *tray_menu;
	GMainLoop *loop;
	
	GtkWidget *StatusWindow;
	GtkWidget *StatusLabel;
	GtkWidget *StatusRssi;
	GtkWidget *StatusRssi2;
	GtkWidget *StatusCinr;
	GtkWidget *StatusCinr2;
	GtkWidget *StatusTxPwr;
	GtkWidget *StatusFreq;
	GtkWidget *StatusBSID;
	GtkWidget *StatusButtonConnect;
	
	GtkWidget *InfoWindow;
	GtkWidget *InfoDevChip;
	GtkWidget *InfoDevSWVer;
	GtkWidget *InfoDevMAC;
	
	GtkWidget *ConfigWindow;
	
	GtkWidget *nspid;
	GtkWidget *use_pkm;
	GtkWidget *eap_type;
	GtkWidget *anonymous_identity;
	GtkWidget *identity;
	GtkWidget *password;
	GtkWidget *cert_null;
	GtkWidget *cert_nv;
	GtkWidget *ca_cert;
	GtkWidget *client_cert;
	GtkWidget *private_key;
	GtkWidget *private_key_passwd;
	
	GtkWidget *wimax_verbose_level;
	GtkWidget *wpa_debug_level;
	GtkWidget *log_file;
	GtkWidget *event_script;
};

static void setDefaultLabels(struct wimax_cm *);
static void setConfigFile(struct wimax_cm *);
static void getConfigFileParam(struct wimax_cm *);

static
void destroy( GtkWidget *widget, struct wimax_cm *wcm)
{
	int status;
	printf("Kill child: %d\n",kill(wcm->child_pid, SIGINT));
	wait3(&status, WNOHANG, NULL);
	printf("Child exited with status %d\n", status);
	
	notify_uninit ();
	g_key_file_free (wcm->keyfile);
	g_main_loop_quit (wcm->loop);
}


static
void tray_icon_on_click(GtkStatusIcon *status_icon,gpointer window)
{
        printf("Clicked on tray icon\n");
	gtk_widget_show(GTK_WIDGET(window));
   	gtk_window_deiconify(GTK_WINDOW(window));    
}


static
void tray_icon_on_menu(GtkStatusIcon *status_icon, guint button,guint activate_time, gpointer popUpMenu)
{
	printf("Popup menu\n");
	
	gtk_menu_popup(	GTK_MENU(popUpMenu), 
					NULL, 
					NULL, 
					gtk_status_icon_position_menu, 
					status_icon, 
					button, 
					activate_time);	
}


static 
gboolean window_state_event (GtkWidget *widget, GdkEventWindowState *event, struct wimax_cm *wcm)
{
    if	(event->changed_mask == GDK_WINDOW_STATE_ICONIFIED && 
		(event->new_window_state == GDK_WINDOW_STATE_ICONIFIED || 
		 event->new_window_state == (GDK_WINDOW_STATE_ICONIFIED | GDK_WINDOW_STATE_MAXIMIZED)))
    {
        gtk_widget_hide (GTK_WIDGET(widget));

    }
    else if	(event->changed_mask == GDK_WINDOW_STATE_WITHDRAWN && 
			(event->new_window_state == GDK_WINDOW_STATE_ICONIFIED || 
			 event->new_window_state == (GDK_WINDOW_STATE_ICONIFIED | GDK_WINDOW_STATE_MAXIMIZED)))
    {
    }
    return TRUE;
}


static
void makeTrayNotify(struct wimax_cm *wcm)
{
	wcm->tray_notify = notify_notification_new ("gctWiMAX CM", NULL, NULL, NULL);
	notify_notification_attach_to_status_icon (wcm->tray_notify, wcm->tray_icon);
	notify_notification_set_timeout(wcm->tray_notify,3000);
	char category[30] = "gctwimax";
	notify_notification_set_category(wcm->tray_notify, category);
	notify_notification_set_urgency (wcm->tray_notify, NOTIFY_URGENCY_CRITICAL);
}


static 
void notify_show(struct wimax_cm *wcm, char *msg)
{
//	char name[100] = msg;	//"Sample Notification"
//	char name[40] = "Sample Notification";


	GError *errorG = NULL;
	// Close previous notification
//	if (example)
	notify_notification_close (wcm->tray_notify, NULL);
	notify_notification_update(wcm->tray_notify, msg, NULL, NULL);
/*
	GtkStatusIcon *status_icon = wcm->tray_icon;
	NotifyNotification *example = wcm->tray_notify;	
	example = notify_notification_new (msg, NULL, NULL, NULL); //"Checking it out"
	notify_notification_attach_to_status_icon (example, status_icon);
	notify_notification_set_timeout(example,3000);
	char category[30] = "gctwimax";
	notify_notification_set_category(example,category);
	notify_notification_set_urgency (example,NOTIFY_URGENCY_CRITICAL);		
*/	
	notify_notification_show(wcm->tray_notify,&errorG);		
}

static DBusHandlerResult
signal_filter (DBusConnection *connection, DBusMessage *message, void *data)
{
	DBusError error;
	struct wimax_cm *wcm = data;
	
	dbus_error_init (&error);

	if (dbus_message_is_signal (message, "ua.org.yarx.Daemon", "SendConnectionInfo")) {
		short state;
		char *msg_state;
		
		if (dbus_message_get_args( message, &error,
						DBUS_TYPE_INT16, &state,
						DBUS_TYPE_STRING, &msg_state, 
						DBUS_TYPE_INVALID)) {
			g_print("SendConnectionInfo received: %s\n",msg_state);
			gtk_label_set_text(GTK_LABEL(wcm->StatusLabel), msg_state);
			notify_show(wcm, msg_state);
		} else 
			goto error_msg;
		goto handled_msg;
	}
	else if (dbus_message_is_signal (message, "ua.org.yarx.Daemon", "SendSignalInfo")) {
		short rssi1;
		short rssi2;
		short cinr1;
		short cinr2;
		unsigned short txpwr;
		unsigned int freq;
				
		if (dbus_message_get_args( message, &error,
						DBUS_TYPE_INT16, &rssi1,
						DBUS_TYPE_INT16, &rssi2,
						DBUS_TYPE_INT16, &cinr1,
						DBUS_TYPE_INT16, &cinr2,
						DBUS_TYPE_UINT16, &txpwr,
						DBUS_TYPE_UINT32, &freq,		
						DBUS_TYPE_INVALID)) {
						
			char str_rssi1[10];			
			char str_rssi2[10];			
			char str_cinr1[10];			
			char str_cinr2[10];			
			char str_txpwr[10];			
			char str_freq[10];			
			sprintf(str_rssi1, "%hddBm", rssi1);
			sprintf(str_rssi2, "%hddBm", rssi2);
			sprintf(str_cinr1, "%hddB", cinr1);
			sprintf(str_cinr2, "%hddB", cinr2);
			sprintf(str_txpwr, "%hddBm", txpwr);
			sprintf(str_freq, "%dkHz", freq);
			
			gtk_label_set_text(GTK_LABEL(wcm->StatusRssi), str_rssi1);
			gtk_label_set_text(GTK_LABEL(wcm->StatusRssi2), str_rssi2);
			gtk_label_set_text(GTK_LABEL(wcm->StatusCinr), str_cinr1);
			gtk_label_set_text(GTK_LABEL(wcm->StatusCinr2), str_cinr2);
			gtk_label_set_text(GTK_LABEL(wcm->StatusTxPwr), str_txpwr);
			gtk_label_set_text(GTK_LABEL(wcm->StatusFreq), str_freq);
			
		} else 
			goto error_msg;
		goto handled_msg;	
	}
	else if (dbus_message_is_signal (message, "ua.org.yarx.Daemon", "SendBsidInfo")) {
		char *bsid;
		if (dbus_message_get_args( message, &error,
						DBUS_TYPE_STRING, &bsid,		
						DBUS_TYPE_INVALID)) {
			gtk_label_set_text(GTK_LABEL(wcm->StatusBSID), bsid);
			gtk_label_set_text(GTK_LABEL(wcm->StatusLabel), "Connected");
		} else 
			goto error_msg;
		goto handled_msg;
	}
	else if (dbus_message_is_signal (message, "ua.org.yarx.Daemon", "SendDeviceInfo")) {
	
		char *chip;
		char *firmware;
		char *mac;

		if (dbus_message_get_args( message, &error,
						DBUS_TYPE_STRING, &chip,
						DBUS_TYPE_STRING, &firmware,
						DBUS_TYPE_STRING, &mac,		
						DBUS_TYPE_INVALID)) {
			g_print("SendDeviceInfo received: %s\n", chip);
			gtk_label_set_text(GTK_LABEL(wcm->InfoDevChip), chip);
			gtk_label_set_text(GTK_LABEL(wcm->InfoDevSWVer), firmware);
			gtk_label_set_text(GTK_LABEL(wcm->InfoDevMAC), mac);
			
		} else 
			goto error_msg;
		goto handled_msg;
	}
	else if (dbus_message_is_signal (message, "ua.org.yarx.Daemon", "ErrorAndDisconnect")) {
		gtk_label_set_text(GTK_LABEL(wcm->StatusLabel), "Disconnected");
		wcm->child_pid = 0;
		notify_show(wcm, "Disconnected");
		setDefaultLabels(wcm);
		gtk_button_set_label (GTK_BUTTON (wcm->StatusButtonConnect), "Connect");
	}
no_msg:	
	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
error_msg:
	g_print("Msg received, but error getting message: %s\n", error.message);
	dbus_error_free (&error);
	goto no_msg;
handled_msg:
	return DBUS_HANDLER_RESULT_HANDLED;	
}


static 
void trayInfo(GtkMenuItem *item, gpointer window) 
{
    gtk_widget_show_all(GTK_WIDGET(window));
    gtk_window_deiconify(GTK_WINDOW(window));    
}

static 
void trayStatus(GtkMenuItem *item, gpointer window) 
{
    gtk_widget_show(GTK_WIDGET(window));
    gtk_window_deiconify(GTK_WINDOW(window));    
}


static 
void trayConfig(GtkMenuItem *item, gpointer window) 
{
    gtk_widget_show_all(GTK_WIDGET(window));
    gtk_window_deiconify(GTK_WINDOW(window));    
}


static
void ButtonConnect(GtkWidget *button, struct wimax_cm *wcm)
{
	if(!strncmp(gtk_button_get_label (GTK_BUTTON (wcm->StatusButtonConnect)), "C", 1))
	{	
		int pid = fork();
		printf("Connnect\n");
		if(pid < 0) { /* error */
			return;
		} else if (pid > 0) { /* parent */
			gtk_label_set_text(GTK_LABEL(wcm->StatusLabel), "");
			gtk_button_set_label (GTK_BUTTON (wcm->StatusButtonConnect), "Disconnect");
			wcm->child_pid = pid;
			printf("child pid: %d\n",pid);
			return;
		} else { /* child */
			int len = strlen(wcm->cfgfile)+12;
			char cfgfile[len];
			snprintf(cfgfile,len,"--cfg-file=%s",wcm->cfgfile);
			char *args[] = {"/sbin/gctwimax", cfgfile, "--with-dbus", NULL};
			char *env[] = {"DISPLAY=:0",NULL};
			/* run the program */
			execve(args[0], args, env);
			exit(1);
		}
	}
	else {
		if (wcm->child_pid != 0)
		{
			kill(wcm->child_pid, SIGINT);
			wcm->child_pid = 0;
		}
		else {
			gtk_button_set_label (GTK_BUTTON (wcm->StatusButtonConnect), "Connect");
		}
		printf("label: %s\n",gtk_button_get_label (GTK_BUTTON (wcm->StatusButtonConnect)));
	}
}


static
void makeStatusWindow(struct wimax_cm *wcm)
{
//	GtkWidget *StatusWindow = wcm->StatusWindow;
	GtkWidget *tableStatus;
	GtkWidget *labelRssiCinr;
	GtkWidget *labelRssiCinr2;
	GtkWidget *labelTxpwrFreq;
	GtkWidget *labelBSID;
	
    wcm->StatusWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (wcm->StatusWindow), "gctWiMAX connection manager");
	gtk_container_set_border_width (GTK_CONTAINER (wcm->StatusWindow), 5);
	gtk_window_set_default_size(GTK_WINDOW(wcm->StatusWindow), 290, 200);
	gtk_window_set_position (GTK_WINDOW (wcm->StatusWindow),GTK_WIN_POS_CENTER);
//	gtk_widget_set_size_request(window, 100, 100);
	gtk_window_set_resizable (GTK_WINDOW (wcm->StatusWindow), FALSE);
	gtk_window_set_deletable (GTK_WINDOW (wcm->StatusWindow), FALSE);	
	wcm->child_pid = 0;
	wcm->StatusButtonConnect = gtk_button_new_with_label("Connect");
	gtk_widget_set_size_request(wcm->StatusButtonConnect, 40, 30);
	
	wcm->StatusLabel = gtk_label_new("");
	wcm->StatusRssi = gtk_label_new("");
	wcm->StatusRssi2 = gtk_label_new("");
	wcm->StatusCinr = gtk_label_new("");
	wcm->StatusCinr2 = gtk_label_new("");
	wcm->StatusTxPwr = gtk_label_new("");
	wcm->StatusFreq = gtk_label_new("");
	wcm->StatusBSID = gtk_label_new("");
	
	labelRssiCinr = gtk_label_new("RSSI/CINR:");
	labelRssiCinr2 = gtk_label_new("RSSI2/CINR2:");
	labelTxpwrFreq = gtk_label_new("TxPwr/Freq:");
	labelBSID = gtk_label_new("BSID:");
	
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusLabel),0.5,0.5);
	
	gtk_misc_set_alignment(GTK_MISC(labelRssiCinr),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusRssi),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusCinr),0.0,0.5);
	
	gtk_misc_set_alignment(GTK_MISC(labelRssiCinr2),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusRssi2),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusCinr2),0.0,0.5);	
	
	gtk_misc_set_alignment(GTK_MISC(labelTxpwrFreq),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusTxPwr),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusFreq),0.0,0.5);	
	
	gtk_misc_set_alignment(GTK_MISC(labelBSID),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->StatusBSID),0.5,0.5);
	
	
	tableStatus = gtk_table_new(6, 3, TRUE);
//	gtk_widget_set_size_request(tableStatus, 100, 100);
	gtk_table_set_row_spacings(GTK_TABLE(tableStatus), 5);
	gtk_table_set_col_spacings(GTK_TABLE(tableStatus), 5);	
	
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusLabel, 0, 3, 0, 1 );
	
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), labelRssiCinr, 	0, 1, 1, 2 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusRssi, 	1, 2, 1, 2 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusCinr, 	2, 3, 1, 2 );
	
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), labelRssiCinr2, 	0, 1, 2, 3 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusRssi2, 1, 2, 2, 3 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusCinr2, 2, 3, 2, 3 );
	
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), labelTxpwrFreq, 	0, 1, 3, 4 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusTxPwr, 1, 2, 3, 4 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusFreq, 	2, 3, 3, 4 );
	
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), labelBSID, 		0, 1, 4, 5 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusBSID, 	1, 3, 4, 5 );
	gtk_table_attach_defaults(GTK_TABLE(tableStatus), wcm->StatusButtonConnect, 	0, 3, 5, 6 );
	gtk_container_add(GTK_CONTAINER(wcm->StatusWindow), tableStatus);
	
	g_signal_connect(G_OBJECT(wcm->StatusButtonConnect), "clicked", G_CALLBACK(ButtonConnect), wcm);
	g_signal_connect(G_OBJECT(wcm->StatusWindow), "destroy",G_CALLBACK (destroy), &wcm);
	g_signal_connect(G_OBJECT(wcm->StatusWindow), "window-state-event", G_CALLBACK (window_state_event), wcm->tray_icon);
	 
	gtk_widget_show_all (tableStatus);
}


static
void makeInfoWindow(struct wimax_cm *wcm)
{
	GtkWidget *tableInfo;
	GtkWidget *tableInfoValign;
	GtkWidget *labelDevChip = gtk_label_new("Device chip:");
	GtkWidget *labelDevSWVer = gtk_label_new("Device SW Version:");
	GtkWidget *labelDevMAC = gtk_label_new("MAC-address:");

    wcm->InfoWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (wcm->InfoWindow), "gctWiMAX device Info");	
	gtk_container_set_border_width (GTK_CONTAINER (wcm->InfoWindow), 5);
	gtk_window_set_position (GTK_WINDOW (wcm->InfoWindow),GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (wcm->InfoWindow), FALSE);
	gtk_window_set_deletable (GTK_WINDOW (wcm->InfoWindow), FALSE);
	
	wcm->InfoDevChip = gtk_label_new("");
	wcm->InfoDevSWVer = gtk_label_new("");
	wcm->InfoDevMAC = gtk_label_new("");
	
	gtk_misc_set_alignment(GTK_MISC(labelDevChip),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(labelDevSWVer),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(labelDevMAC),1.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->InfoDevChip),0.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->InfoDevSWVer),0.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(wcm->InfoDevMAC),0.0,0.5);
	
	tableInfo = gtk_table_new(3, 2, TRUE);
	gtk_table_set_row_spacings(GTK_TABLE(tableInfo), 5);
	gtk_table_set_col_spacings(GTK_TABLE(tableInfo), 5);

	gtk_table_attach_defaults(GTK_TABLE(tableInfo), labelDevChip, 	0, 1, 0, 1 );
	gtk_table_attach_defaults(GTK_TABLE(tableInfo), labelDevSWVer, 	0, 1, 1, 2 );
	gtk_table_attach_defaults(GTK_TABLE(tableInfo), labelDevMAC, 	0, 1, 2, 3 );
	gtk_table_attach_defaults(GTK_TABLE(tableInfo), wcm->InfoDevChip, 	1, 2, 0, 1 );
	gtk_table_attach_defaults(GTK_TABLE(tableInfo), wcm->InfoDevSWVer, 	1, 2, 1, 2 );
	gtk_table_attach_defaults(GTK_TABLE(tableInfo), wcm->InfoDevMAC, 	1, 2, 2, 3 );
	
	tableInfoValign = gtk_alignment_new(0.5, 0.5, 0, 0);
	gtk_container_add(GTK_CONTAINER(wcm->InfoWindow), tableInfoValign);
	gtk_container_add(GTK_CONTAINER(tableInfoValign), tableInfo);
	
	g_signal_connect (G_OBJECT (wcm->InfoWindow), "window-state-event", G_CALLBACK (window_state_event), wcm);
}


static
void text_digit_check_handler (GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
  GtkEditable *editable = GTK_EDITABLE(entry);
  int i, count=0;
  gchar *result = g_new (gchar, length);

  for (i=0; i < length; i++) {
    if (!isdigit(text[i]))
      continue;
    result[count++] = text[i]; //islower(text[i]) ? toupper(text[i]) : 
  }
  
  if (count > 0) {
    g_signal_handlers_block_by_func (G_OBJECT (editable),
                                     G_CALLBACK (text_digit_check_handler),
                                     data);
    gtk_editable_insert_text (editable, result, count, position);
    g_signal_handlers_unblock_by_func (G_OBJECT (editable),
                                       G_CALLBACK (text_digit_check_handler),
                                       data);
  }
  g_signal_stop_emission_by_name (G_OBJECT (editable), "insert_text");

  g_free (result);
}


static
void toogle_cert_nv(GtkWidget *widget, struct wimax_cm *wcm)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->ca_cert), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->client_cert), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->private_key), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->private_key_passwd), FALSE);		
	} else {
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->ca_cert), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->client_cert), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->private_key), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->private_key_passwd), TRUE);		
	}
}


static
void toogle_cert_null(GtkWidget *widget, struct wimax_cm *wcm)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->cert_nv), FALSE);
		toogle_cert_nv(widget, wcm);
	} else {
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->cert_nv), TRUE);
		toogle_cert_nv(wcm->cert_nv, wcm);
	}
}


static
void toogle_use_pkm(GtkWidget *widget, struct wimax_cm *wcm)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->eap_type), TRUE);	
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->anonymous_identity), TRUE);	
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->identity), TRUE);	
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->password), TRUE);	
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->cert_null), TRUE);
		toogle_cert_null(wcm->cert_null, wcm);
	} else {
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->eap_type), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->anonymous_identity), FALSE);	
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->identity), FALSE);	
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->password), FALSE);	
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->cert_null), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->cert_nv), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->ca_cert), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->client_cert), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->private_key), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET(wcm->private_key_passwd), FALSE);
	}
}


static
void setDefaultConfig(struct wimax_cm *wcm)
{
	gchar nspid[20];
	sprintf(nspid, "%d", wcm->conf->nspid);
	gtk_entry_set_text(GTK_ENTRY(wcm->nspid), nspid);
	
	if (wcm->conf->use_pkm == 1)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wcm->use_pkm), TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wcm->use_pkm), FALSE);
	

	if (wcm->conf->eap_type == 5)
		gtk_combo_box_set_active (GTK_COMBO_BOX(wcm->eap_type), 1);
	else if (wcm->conf->eap_type == 3)
		gtk_combo_box_set_active (GTK_COMBO_BOX(wcm->eap_type), 0);
	else
		gtk_combo_box_set_active (GTK_COMBO_BOX(wcm->eap_type), -1);
		
	if(wcm->conf->anonymous_identity)
		gtk_entry_set_text(GTK_ENTRY(wcm->anonymous_identity), wcm->conf->anonymous_identity);
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->anonymous_identity), "");		
	if(wcm->conf->identity)		
		gtk_entry_set_text(GTK_ENTRY(wcm->identity), wcm->conf->identity);
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->identity), "");
	if(wcm->conf->password)	
		gtk_entry_set_text(GTK_ENTRY(wcm->password), wcm->conf->password); 
	else		
		gtk_entry_set_text(GTK_ENTRY(wcm->password), ""); 
		
	if (wcm->conf->cert_null == 1)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wcm->cert_null), TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wcm->cert_null), FALSE);

	if (wcm->conf->cert_nv == 1)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wcm->cert_nv), TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wcm->cert_nv), FALSE);		
	
	if(wcm->conf->ca_cert)
		gtk_entry_set_text(GTK_ENTRY(wcm->ca_cert), wcm->conf->ca_cert);
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->ca_cert), "");
	if(wcm->conf->client_cert)
		gtk_entry_set_text(GTK_ENTRY(wcm->client_cert), wcm->conf->client_cert);
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->client_cert), "");
	if(wcm->conf->private_key)
		gtk_entry_set_text(GTK_ENTRY(wcm->private_key), wcm->conf->private_key);
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->private_key), "");
	if(wcm->conf->private_key_passwd)
		gtk_entry_set_text(GTK_ENTRY(wcm->private_key_passwd), wcm->conf->private_key_passwd); 
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->private_key_passwd), "");


	if (wcm->conf->wimax_verbose_level <= 3 || wcm->conf->wimax_verbose_level >= -1)
		gtk_combo_box_set_active (GTK_COMBO_BOX(wcm->wimax_verbose_level), wcm->conf->wimax_verbose_level + 1);
	else
		gtk_combo_box_set_active (GTK_COMBO_BOX(wcm->wimax_verbose_level), 1);
	if (wcm->conf->wpa_debug_level <= 4 || wcm->conf->wpa_debug_level >= 0)
		gtk_combo_box_set_active (GTK_COMBO_BOX(wcm->wpa_debug_level), wcm->conf->wpa_debug_level);
	else
		gtk_combo_box_set_active (GTK_COMBO_BOX(wcm->wpa_debug_level), 3);		
	if(wcm->conf->log_file)
		gtk_entry_set_text(GTK_ENTRY(wcm->log_file), wcm->conf->log_file); 
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->log_file), "");
	if(wcm->conf->event_script)
		gtk_entry_set_text(GTK_ENTRY(wcm->event_script), wcm->conf->event_script); 
	else
		gtk_entry_set_text(GTK_ENTRY(wcm->event_script), "");
		
	toogle_cert_null(wcm->cert_null, wcm);
	toogle_use_pkm(wcm->use_pkm, wcm);
}


static
void ButtonCancel(GtkWidget *button, struct wimax_cm *wcm)
{
	setDefaultConfig(wcm);
	gtk_widget_hide(wcm->ConfigWindow);
}


static
void ButtonApply(GtkWidget *button, struct wimax_cm *wcm)
{
	setConfigFile(wcm);
	gtk_widget_hide(wcm->ConfigWindow);
	if(wcm->child_pid != 0)
		ButtonConnect(wcm->StatusButtonConnect, wcm);
	ButtonConnect(wcm->StatusButtonConnect, wcm);
}


static
void file_set(GtkWidget *file, struct wimax_cm *wcm)
{
	gchar *newfile = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (file));
	printf("selected new >%s<\n",newfile);

	GError *error = NULL;
	
	if (!g_key_file_load_from_file (wcm->keyfile, newfile, G_KEY_FILE_KEEP_COMMENTS, &error))
	{
		printf("Unable to read file: %s\n", error->message);
		g_error_free(error);
		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file), wcm->cfgfile);
		return;
	}	
	wcm->cfgfile = newfile;
	getConfigFileParam(wcm);
	setDefaultConfig(wcm);
}


static
void makeConfigWindow(struct wimax_cm *wcm)
{
    wcm->ConfigWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (wcm->ConfigWindow), "gctWiMAX Configuration");	
	gtk_container_set_border_width (GTK_CONTAINER (wcm->ConfigWindow), 5);
	gtk_window_set_position (GTK_WINDOW (wcm->ConfigWindow),GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (wcm->ConfigWindow), FALSE);
	gtk_window_set_deletable (GTK_WINDOW (wcm->ConfigWindow), FALSE);
	
	GtkWidget *file;
	GtkFileFilter *filter = gtk_file_filter_new ();
	
	GtkWidget *tableConfig = gtk_table_new(16, 2, TRUE);
	GtkWidget *nspid = wcm->nspid = gtk_entry_new();
	GtkWidget *use_pkm = wcm->use_pkm = gtk_check_button_new_with_label("Use PKM");
	GtkWidget *eap_type = wcm->eap_type = gtk_combo_box_new_text();
	GtkWidget *anonymous_identity = wcm->anonymous_identity = gtk_entry_new();
	GtkWidget *identity = wcm->identity = gtk_entry_new();
	GtkWidget *password = wcm->password = gtk_entry_new();
	GtkWidget *cert_null = wcm->cert_null = gtk_check_button_new_with_label("Cert Null");
	GtkWidget *cert_nv = wcm->cert_nv = gtk_check_button_new_with_label("Cert from NV");
	GtkWidget *ca_cert = wcm->ca_cert = gtk_entry_new();
	GtkWidget *client_cert = wcm->client_cert = gtk_entry_new();
	GtkWidget *private_key = wcm->private_key = gtk_entry_new();
	GtkWidget *private_key_passwd = wcm->private_key_passwd = gtk_entry_new();
	
	GtkWidget *wimax_verbose_level = wcm->wimax_verbose_level = gtk_combo_box_new_text();
	GtkWidget *wpa_debug_level = wcm->wpa_debug_level = gtk_combo_box_new_text();
	GtkWidget *log_file = wcm->log_file = gtk_entry_new();
	GtkWidget *event_script = wcm->event_script = gtk_entry_new();
	
	GtkWidget *ConfigButtonCancel = gtk_button_new_with_label("Cancel");
	GtkWidget *ConfigButtonApply = gtk_button_new_with_label("Apply");
	
	GtkWidget *use_pkm_align = gtk_alignment_new(0.5, 0.5, 0, 0);
	GtkWidget *cert_null_align = gtk_alignment_new(0.5, 0.5, 0, 0);
	GtkWidget *cert_nv_align = gtk_alignment_new(0.5, 0.5, 0, 0);
	gtk_container_add(GTK_CONTAINER(use_pkm_align), use_pkm);
	gtk_container_add(GTK_CONTAINER(cert_null_align), cert_null);
	gtk_container_add(GTK_CONTAINER(cert_nv_align), cert_nv);
	
	gtk_combo_box_append_text(GTK_COMBO_BOX(eap_type), "TLS");
	gtk_combo_box_append_text(GTK_COMBO_BOX(eap_type), "TTLS_MSCHAPV2");
	
	gtk_entry_set_visibility (GTK_ENTRY(password), FALSE);
	gtk_entry_set_visibility (GTK_ENTRY(private_key_passwd), FALSE);
	
	gtk_combo_box_append_text(GTK_COMBO_BOX(wimax_verbose_level), "-1");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wimax_verbose_level), "0");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wimax_verbose_level), "1");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wimax_verbose_level), "2");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wimax_verbose_level), "3");
	
	gtk_combo_box_append_text(GTK_COMBO_BOX(wpa_debug_level), "0");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wpa_debug_level), "1");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wpa_debug_level), "2");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wpa_debug_level), "3");
	gtk_combo_box_append_text(GTK_COMBO_BOX(wpa_debug_level), "4");

	setDefaultConfig(wcm);
	
	g_signal_connect(G_OBJECT(use_pkm), "clicked",G_CALLBACK(toogle_use_pkm), wcm);	
	g_signal_connect(G_OBJECT(cert_null), "clicked",G_CALLBACK(toogle_cert_null), wcm);
	g_signal_connect(G_OBJECT(cert_nv), "clicked",G_CALLBACK(toogle_cert_nv), wcm);

	gtk_table_set_row_spacings(GTK_TABLE(tableConfig), 0);
	gtk_table_set_col_spacings(GTK_TABLE(tableConfig), 0);
	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("NSP ID:"), 	0, 1, 0, 1 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), nspid, 	1, 2, 0, 1 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), use_pkm_align, 	0, 1, 1, 2 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), eap_type, 	1, 2, 1, 2 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Anonymous:"), 	0, 1, 2, 3 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), anonymous_identity, 	1, 2, 2, 3 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Identity:"), 	0, 1, 3, 4 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), identity, 	1, 2, 3, 4 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Password:"), 	0, 1, 4, 5 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), password, 	1, 2, 4, 5 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), cert_null_align, 	0, 1, 5, 6 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), cert_nv_align, 	1, 2, 5, 6 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("CA Cert:"), 	0, 1, 6, 7 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), ca_cert, 	1, 2, 6, 7 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Client Cert:"), 	0, 1, 7, 8 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), client_cert, 	1, 2, 7, 8 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Private key:"), 	0, 1, 8, 9 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), private_key, 	1, 2, 8, 9 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Private key pass:"), 	0, 1, 9, 10 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), private_key_passwd, 	1, 2, 9, 10 );
	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Wimax log level:"), 	0, 1, 10, 11 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), wimax_verbose_level, 	1, 2, 10, 11 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("EAP debug level:"), 	0, 1, 11, 12 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), wpa_debug_level, 	1, 2, 11, 12 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Log FILE path:"), 	0, 1, 12, 13 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), log_file, 	1, 2, 12, 13 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), gtk_label_new("Event script path:"), 	0, 1, 13, 14 );	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), event_script, 	1, 2, 13, 14 );	
	
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), ConfigButtonCancel, 	0, 1, 14, 15 );
	gtk_table_attach_defaults(GTK_TABLE(tableConfig), ConfigButtonApply, 	1, 2, 14, 15 );

	file = gtk_file_chooser_button_new ("Select config file", GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file), wcm->cfgfile);
	gtk_file_filter_add_pattern (filter,"*.conf");
	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file), filter);

	gtk_table_attach_defaults(GTK_TABLE(tableConfig), file, 	0, 2, 15, 16 );
	
	gtk_container_add(GTK_CONTAINER(wcm->ConfigWindow), tableConfig);
	
	g_signal_connect(G_OBJECT(nspid), "insert_text", G_CALLBACK(text_digit_check_handler), NULL);
	g_signal_connect(G_OBJECT(ConfigButtonCancel), "clicked", G_CALLBACK(ButtonCancel), wcm);
	g_signal_connect(G_OBJECT(ConfigButtonApply), "clicked", G_CALLBACK(ButtonApply), wcm);
	g_signal_connect(G_OBJECT (wcm->ConfigWindow), "window-state-event", G_CALLBACK (window_state_event), wcm);
	g_signal_connect(G_OBJECT(file), "file-set", G_CALLBACK(file_set), wcm);
	
	gtk_widget_show_all (tableConfig);
}


static
void setDefaultLabels(struct wimax_cm *wcm)
{
	gtk_label_set_text(GTK_LABEL(wcm->StatusRssi), 	"-00dBm");
	gtk_label_set_text(GTK_LABEL(wcm->StatusRssi2), "-00dBm");
	gtk_label_set_text(GTK_LABEL(wcm->StatusCinr), 	"00dB");
	gtk_label_set_text(GTK_LABEL(wcm->StatusCinr2), "00dB");
	gtk_label_set_text(GTK_LABEL(wcm->StatusTxPwr), "00dBm");
	gtk_label_set_text(GTK_LABEL(wcm->StatusFreq), 	"0000000kHz");
	gtk_label_set_text(GTK_LABEL(wcm->StatusBSID), 	"00:00:00:00:00:00");
	
	gtk_label_set_text(GTK_LABEL(wcm->InfoDevChip), "");
	gtk_label_set_text(GTK_LABEL(wcm->InfoDevSWVer), "");
	gtk_label_set_text(GTK_LABEL(wcm->InfoDevMAC), "");	
}


static
void makeTrayIcon(struct wimax_cm *wcm)
{
	wcm->tray_icon = gtk_status_icon_new();
	
	gtk_status_icon_set_from_icon_name(wcm->tray_icon,GTK_STOCK_NETWORK);
	gtk_status_icon_set_tooltip(wcm->tray_icon,"GCTCM");
	gtk_status_icon_set_visible(wcm->tray_icon, TRUE);	
	
	g_signal_connect(G_OBJECT(wcm->tray_icon), "activate",G_CALLBACK(tray_icon_on_click), wcm->StatusWindow);
	g_signal_connect(G_OBJECT(wcm->tray_icon), "popup-menu",G_CALLBACK(tray_icon_on_menu), wcm->tray_menu);	
}


static
void makeTrayMenu(struct wimax_cm *wcm)
{
	GtkWidget *menuItemInfo, *menuItemStatus, *menuItemConfig, *menuItemExit;
	
	wcm->tray_menu = gtk_menu_new();
	menuItemStatus = gtk_menu_item_new_with_label ("Status");
	menuItemInfo = gtk_menu_item_new_with_label ("Device Info");
	menuItemConfig = gtk_menu_item_new_with_label ("Configuration");
	menuItemExit = gtk_menu_item_new_with_label ("Exit");
	
	g_signal_connect (G_OBJECT (menuItemStatus),"activate", G_CALLBACK (trayStatus), wcm->StatusWindow);
	g_signal_connect (G_OBJECT (menuItemInfo), 	"activate", G_CALLBACK (trayInfo), wcm->InfoWindow);
	g_signal_connect (G_OBJECT (menuItemConfig),"activate", G_CALLBACK (trayConfig), wcm->ConfigWindow);
	g_signal_connect (G_OBJECT (menuItemExit), 	"activate", G_CALLBACK (destroy), wcm);
	gtk_menu_shell_append (GTK_MENU_SHELL (wcm->tray_menu), menuItemStatus);
	gtk_menu_shell_append (GTK_MENU_SHELL (wcm->tray_menu), menuItemInfo);
	gtk_menu_shell_append (GTK_MENU_SHELL (wcm->tray_menu), menuItemConfig);
	gtk_menu_shell_append (GTK_MENU_SHELL (wcm->tray_menu), menuItemExit);
	gtk_widget_show_all (wcm->tray_menu);
}


static
gchar *remQuotation(gchar *str)
{
	if (str != NULL){
		if ( str[0] == '"'){
			int length = strlen(str);
			if ( str[length-1] == '"'){
				gchar *buf = g_new (gchar, length - 1);
				memcpy(buf, str+1, length - 2);
				buf[length-2] = '\0';
				str = buf;
			}
		}
	}
	return str;
}
	

static
void getConfigFileParam(struct wimax_cm *wcm)
{
	wcm->conf->nspid = g_key_file_get_integer (wcm->keyfile, "common", "nspid", NULL);
	wcm->conf->use_pkm = g_key_file_get_integer (wcm->keyfile, "auth", "use_pkm", NULL);
	wcm->conf->eap_type = g_key_file_get_integer (wcm->keyfile, "auth", "eap_type", NULL);
	wcm->conf->cert_null = g_key_file_get_integer (wcm->keyfile, "auth", "cert_null", NULL);
	wcm->conf->cert_nv = g_key_file_get_integer (wcm->keyfile, "auth", "cert_nv", NULL);
	wcm->conf->anonymous_identity = remQuotation(g_key_file_get_string (wcm->keyfile, "auth", "anonymous_identity", NULL));
	wcm->conf->identity = remQuotation(g_key_file_get_string (wcm->keyfile, "auth", "identity", NULL));
	wcm->conf->password = remQuotation(g_key_file_get_string (wcm->keyfile, "auth", "password", NULL));
	wcm->conf->ca_cert = remQuotation(g_key_file_get_string (wcm->keyfile, "auth", "ca_cert", NULL));
	wcm->conf->client_cert = remQuotation(g_key_file_get_string (wcm->keyfile, "auth", "client_cert", NULL));
	wcm->conf->private_key = remQuotation(g_key_file_get_string (wcm->keyfile, "auth", "private_key", NULL));
	wcm->conf->private_key_passwd = remQuotation(g_key_file_get_string (wcm->keyfile, "auth", "private_key_passwd", NULL));	
	
	wcm->conf->wimax_verbose_level = g_key_file_get_integer (wcm->keyfile, "log", "wimax_verbose_level", NULL);
	wcm->conf->wpa_debug_level = g_key_file_get_integer (wcm->keyfile, "log", "wpa_debug_level", NULL);
	wcm->conf->log_file = remQuotation(g_key_file_get_string (wcm->keyfile, "log", "log_file", NULL));
	wcm->conf->event_script = remQuotation(g_key_file_get_string (wcm->keyfile, "log", "event_script", NULL));
}


static
void getConfigFile(struct wimax_cm *wcm)
{
	GError *error = NULL;
	wcm->keyfile = g_key_file_new ();
	wcm->conf = g_slice_new (struct wimax_conf);
	wcm->cfgfile = "/usr/share/gctwimax/cm.conf";
	if (!g_key_file_load_from_file (wcm->keyfile, wcm->cfgfile, G_KEY_FILE_KEEP_COMMENTS, &error))
	{
		printf("Unable to read file: %s\n", error->message);
		g_error_free(error);
		wcm->conf->nspid = 0;
		//g_error (error->message);
		return;//-1
	}
	getConfigFileParam(wcm);
}


static
gchar *addQuotation(gchar *str)
{
	if (str != NULL){
		int length = strlen(str) + 3;
		gchar *buf = g_new (gchar, length);
		snprintf(buf, length,"\"%s\"", str);
//		g_free (str);
		str = buf;
	}
	return str;
}


static
void setConfigFile(struct wimax_cm *wcm)
{
	wcm->conf->nspid = atoi(gtk_entry_get_text(GTK_ENTRY(wcm->nspid)));	

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wcm->use_pkm)))
		wcm->conf->use_pkm = 1;
	else 
		wcm->conf->use_pkm = 0;

	switch(gtk_combo_box_get_active (GTK_COMBO_BOX(wcm->eap_type)))
	{
		case 0:
			wcm->conf->eap_type = 3;
			break;
		case 1:
			wcm->conf->eap_type = 5;
			break;
		default:
			wcm->conf->eap_type = -1;
	}
		
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wcm->cert_null)))
		wcm->conf->cert_null = 1;
	else 
		wcm->conf->cert_null = 0;

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wcm->cert_nv)))
		wcm->conf->cert_nv = 1;
	else 
		wcm->conf->cert_nv = 0;
		
	wcm->conf->wimax_verbose_level = gtk_combo_box_get_active (GTK_COMBO_BOX(wcm->wimax_verbose_level))-1;
	wcm->conf->wpa_debug_level = gtk_combo_box_get_active (GTK_COMBO_BOX(wcm->wpa_debug_level));
		
	wcm->conf->anonymous_identity = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->anonymous_identity));
	wcm->conf->identity = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->identity));
	wcm->conf->password = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->password));
	wcm->conf->ca_cert = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->ca_cert));
	wcm->conf->client_cert = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->client_cert));
	wcm->conf->private_key = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->private_key));
	wcm->conf->private_key_passwd = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->private_key_passwd));
	
	wcm->conf->log_file = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->log_file));
	wcm->conf->event_script = (gchar *)gtk_entry_get_text(GTK_ENTRY(wcm->event_script));
	
	g_key_file_set_integer(wcm->keyfile, "common", "nspid", wcm->conf->nspid);
	g_key_file_set_integer(wcm->keyfile, "auth", "use_pkm", wcm->conf->use_pkm);
	g_key_file_set_integer(wcm->keyfile, "auth", "eap_type", wcm->conf->eap_type);
	g_key_file_set_string(wcm->keyfile, "auth", "anonymous_identity", addQuotation(wcm->conf->anonymous_identity));
	g_key_file_set_string(wcm->keyfile, "auth", "identity", addQuotation(wcm->conf->identity));
	g_key_file_set_string(wcm->keyfile, "auth", "password", addQuotation(wcm->conf->password));
	g_key_file_set_integer(wcm->keyfile, "auth", "cert_null", wcm->conf->cert_null);
	g_key_file_set_integer(wcm->keyfile, "auth", "cert_nv", wcm->conf->cert_nv);
	g_key_file_set_string(wcm->keyfile, "auth", "ca_cert", addQuotation(wcm->conf->ca_cert));
	g_key_file_set_string(wcm->keyfile, "auth", "client_cert", addQuotation(wcm->conf->client_cert));
	g_key_file_set_string(wcm->keyfile, "auth", "private_key", addQuotation(wcm->conf->private_key));
	g_key_file_set_string(wcm->keyfile, "auth", "private_key_passwd", addQuotation(wcm->conf->private_key_passwd));

	g_key_file_set_integer(wcm->keyfile, "log", "wimax_verbose_level", wcm->conf->wimax_verbose_level);
	g_key_file_set_integer(wcm->keyfile, "log", "wpa_debug_level", wcm->conf->wpa_debug_level);
	g_key_file_set_string(wcm->keyfile, "log", "log_file", addQuotation(wcm->conf->log_file));
	g_key_file_set_string(wcm->keyfile, "log", "event_script", addQuotation(wcm->conf->event_script));
	
	FILE *fp;
	fp = fopen(wcm->cfgfile, "w");
	if(!fp){
		printf("Error open file\n");
		return;
	}
	gsize len;
	fprintf(fp,"%s", g_key_file_to_data ( wcm->keyfile, &len, NULL));
	fclose(fp);
}


int main( int   argc,char *argv[] )
{
	gtk_init (&argc, &argv);
	notify_init("GCTCM");
	
	struct wimax_cm wcm;	
	
	DBusConnection *bus;
	DBusError error;
	
	wcm.loop = g_main_loop_new (NULL, FALSE);
	
    getConfigFile(&wcm);
	makeStatusWindow(&wcm);
	makeInfoWindow(&wcm);
	makeConfigWindow(&wcm);
	setDefaultLabels(&wcm);
	makeTrayMenu(&wcm);
	makeTrayIcon(&wcm);
	makeTrayNotify(&wcm);
		
	dbus_error_init (&error);

	bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
	if (!bus) {
	    g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
		dbus_error_free (&error);
		return 1;
	}
	dbus_connection_setup_with_g_main (bus, NULL);	

	dbus_bus_add_match (bus, "type='signal',interface='ua.org.yarx.Daemon'",&error);
	dbus_connection_add_filter (bus, signal_filter, &wcm, NULL);
	
	if (!wcm.conf->anonymous_identity || !strncmp(wcm.conf->anonymous_identity,"\0",1) || 
		!wcm.conf->identity || !strncmp(wcm.conf->identity,"\0",1) || 
		!wcm.conf->password || !strncmp(wcm.conf->password,"\0",1))
		gtk_widget_show (wcm.ConfigWindow);
	else
		ButtonConnect(wcm.StatusButtonConnect, &wcm);
		// Start application
		
	g_main_loop_run (wcm.loop);
    
    return 0;
}
